/*	Gamma - Generic processing library
 See COPYRIGHT file for authors and license information
 
 Example:		Filter / Delay-line
 Description:	This demonstrates the multiple uses of a delay-line.
 */
#include "examples.h"

const int NUMBER_OF_DELAYS = 10;
float delayAmount[NUMBER_OF_DELAYS];

struct delayVoice
{
	float baseDelay;
	float delayWaver;
	float currentDelay;
	float targetDelay;
	
}

delayVoiceArray[NUMBER_OF_DELAYS];
float delayTime = 0.3;
Delay<float, ipl::Trunc> delay(20.0, 0.2);
AllPass1<> passer;
Biquad<> filt(10, 4);
Biquad<> filtArray[NUMBER_OF_DELAYS];
AllPass1<> passArray[NUMBER_OF_DELAYS];


int cnt = 0;
float putOutArray[NUMBER_OF_DELAYS];
int passUseArray[NUMBER_OF_DELAYS];
int passUseArray2[NUMBER_OF_DELAYS];
int voiceUp = 0;



Accum<> tmr(6);//0.5
void audioCB(AudioIOData& io){
	
	while(io()){
		
		if(tmr()) {
			voiceUp = int((rand()%(10*NUMBER_OF_DELAYS))/10.0);
			//printf("VOICE UP: %i", voiceUp);
			float temp_random = (rand()%100)/100.0; //random numbers from 0.0-0.99
			float toAdd = (temp_random*delayVoiceArray[voiceUp].delayWaver*2)-delayVoiceArray[voiceUp].delayWaver;
			//delayVoiceArray[xx].currentDelay = delayVoiceArray[xx].baseDelay + toAdd;
			delayVoiceArray[voiceUp].targetDelay = delayVoiceArray[voiceUp].baseDelay + toAdd;
			
			if(delayVoiceArray[voiceUp].currentDelay <= 0.0){
				delayVoiceArray[voiceUp].currentDelay = 0.01;
			}
		}
	
		float s = io.in(10); //0
		for(int i = 0;i<NUMBER_OF_DELAYS;i++){
			delayVoiceArray[i].currentDelay = delayVoiceArray[i].currentDelay + 
			((delayVoiceArray[i].targetDelay-delayVoiceArray[i].currentDelay)/(184100.0-((rand()%100)*1500.0))); //22050
			
			delay.delay(delayVoiceArray[i].currentDelay);
			
			putOutArray[i] = passer(filtArray[i](passer(passer(passer(passer(passer(passer(passer(passer(passer(passArray[i](delay(s)))))))))))));
			for(int j = 0;j<i*3;j++){
				putOutArray[i] = passArray[i](passer(putOutArray[i]));
			}
		}
		for(int i = 0;i<NUMBER_OF_DELAYS;i++){
			
			io.out(i) = putOutArray[i];	
		}					  
	}
}

int main(){
	for(int zz = 0; zz<NUMBER_OF_DELAYS;zz++){
		delayVoiceArray[zz].baseDelay = 5.0;//0.5;
		delayVoiceArray[zz].delayWaver = 3.0;//0.35; //+zz*0.2;//0.07;
		delayVoiceArray[zz].currentDelay = delayVoiceArray[zz].baseDelay;//0.5;
		delayVoiceArray[zz].targetDelay = 0.0;
	}
	for(int yy = 0; yy<NUMBER_OF_DELAYS;yy++){
		//filtArray[yy].type(10,4);
		switch(cnt){
			case 0: filtArray[yy].type(Filter::BR); //printf("Band-reject\n"); break;			
			case 1: filtArray[yy].type(Filter::HP); //printf("High-pass\n"); break;
			case 2: filtArray[yy].type(Filter::BP); //printf("Band-pass\n"); break;
			case 3:filtArray[yy].type(Filter::LP); //printf("Low-pass\n"); break;
				
		}
		++cnt %= 1;
		filtArray[yy].set(((rand()%100)*100.0),1);//random numbers up to 10k
		passArray[yy].freq(60000+((rand()%100)*100.0));//1000 worked well
	}
	AudioIO io(128, 44100, audioCB, 0, 2, 1);
	Sync::master().spu(io.framesPerSecond());
	io.deviceOut(3);
	io.deviceIn(3);
	io.channelsOut(16);
	io.channelsIn(16);
	io.start();
	printf("\nPress 'enter' to quit...\n"); getchar();
	return 0;
}
